@@ -45,6 +45,8 @@
 #' @param sapto_taper The taper rate beyond \code{sapto_lower_threshold}.
 #' @param sbto_discount The \code{tax_discount} in \code{\link{small_business_tax_offset}}.
 #' 
+#' @param cgt_discount_rate (numeric(1)) The capital gains tax discount rate, currently 50\%. 
+#' 
 #' @param calc_baseline_tax (logical, default: \code{TRUE}) Should the income tax in \code{baseline_fy} be included as a column in the result?
 #' @param return. What should the function return? One of \code{tax}, \code{sample_file}, or \code{sample_file.int}. 
 #' If \code{tax}, the tax payable under the settings; if \code{sample_file}, the \code{sample_file},
@@ -55,14 +57,21 @@
 #' 
 #' @examples
 #' 
+#' library(data.table)
+#' library(hutils)
+#' 
 #' # With new tax-free threshold of $20,000:
 #' if (requireNamespace("taxstats", quietly = TRUE)) {
 #'   library(taxstats)
+#'   library(magrittr)
+#'      
 #'   model_income_tax(sample_file_1314,
 #'                    "2013-14",
-#'                    ordinary_tax_thresholds = c(0, 20e3, 37e3, 80e3, 180e3))
+#'                    ordinary_tax_thresholds = c(0, 20e3, 37e3, 80e3, 180e3)) %>%
+#'     select_grep("tax", "Taxable_Income")
 #' 
 #' }
+#' 
 #' @export
 
 
@@ -108,6 +117,8 @@ model_income_tax <- function(sample_file,
                              
                              sbto_discount = NULL,
                              
+                             cgt_discount_rate = NULL,
+                             
                              calc_baseline_tax = TRUE,
                              return. = c("sample_file", "tax", "sample_file.int"),
                              clear_tax_cols = TRUE,
@@ -127,6 +138,7 @@ model_income_tax <- function(sample_file,
   
   stopifnot(is.data.table(sample_file))
   sample_file <- copy(sample_file)
+  max.length <- nrow(sample_file)
   .dots.ATO <- sample_file
   sample_file_noms <- copy(names(sample_file))
   
@@ -156,7 +168,7 @@ model_income_tax <- function(sample_file,
       "WRE_uniform_amt", "WRE_self_amt", "WRE_other_amt", "Div_Ded_amt", 
       "Intrst_Ded_amt", "Gift_amt", "Non_emp_spr_amt", "Cost_tax_affairs_amt", 
       "Other_Ded_amt", "Tot_ded_amt", "PP_loss_claimed", "NPP_loss_claimed", 
-      "Rep_frng_ben_amt", "Med_Exp_TO_amt", "Asbl_forgn_source_incm_amt", 
+      "Rep_frng_ben_amt", # "Med_Exp_TO_amt", "Asbl_forgn_source_incm_amt", 
       # "Spouse_adjusted_taxable_inc",
       "Net_fincl_invstmt_lss_amt", "Rptbl_Empr_spr_cont_amt", 
       "Cr_PAYG_ITI_amt", "TFN_amts_wheld_gr_intst_amt", "TFN_amts_wheld_divs_amt", 
@@ -168,6 +180,7 @@ model_income_tax <- function(sample_file,
          paste0(absent_cols, collapse = "\n\t"), ".\n")
   }
   
+  
   income <- sample_file[["Taxable_Income"]]
   max.length <- length(income)
   prohibit_vector_recycling(income, n_dependants, baseline_fy)
@@ -176,11 +189,77 @@ model_income_tax <- function(sample_file,
                         fy.year = baseline_fy,
                         .dots.ATO = .dots.ATO,
                         n_dependants = n_dependants)
+  if (calc_baseline_tax && return. != "tax") {
+    set(sample_file,
+        j = "baseline_tax",
+        value = switch(return.,
+                       "sample_file" = old_tax,
+                       "sample_file.int" = as.integer(old_tax)))
+  }
   
-  if (calc_baseline_tax) {
-    switch(return.,
-           "sample_file" = set(sample_file, j = "baseline_tax", value = old_tax),
-           "sample_file.int" = set(sample_file, j = "baseline_tax", value = as.integer(old_tax)))
+  # Recalculate the taxable income
+  # CG adjustment
+  if (!is.null(cgt_discount_rate)) {
+    if (length(cgt_discount_rate) != 1L) {
+      if (length(cgt_discount_rate) != max.length) {
+        stop("`length(cgt_discount_rate) = ", length(cgt_discount_rate), "`, ", 
+             "yet `nrow(sample_file) = ", max.length, "`. ",
+             "Ensure `cgt_discount_rate` has length one.")
+      }
+    }
+    
+    if (!is.numeric(cgt_discount_rate)) {
+      stop("`cgt_discount_rate` was type ", typeof(cgt_discount_rate), ", ",
+           "but must be numeric. Ensure `cgt_discount_rate`, if used, is numeric.")
+    }
+    
+    extra_Net_CG_amt <- function(DT,
+                                 new_rate = cgt_discount_rate,
+                                 old_rate = 0.5) {
+      # Return the extra amount of capital gains
+      # payable under `new_rate` compared to `old_rate`.
+      # That which will be added to both Net_CG_amt and Taxable_Income
+      # in the resultant sample file.
+      
+      
+      # Use this function to avoid copying
+      
+      Net_CG_amt. <- .subset2(DT, "Net_CG_amt")
+      Tot_CY_CG_amt. <- .subset2(DT, "Tot_CY_CG_amt")
+      income <- .subset2(DT, "Taxable_Income")
+      
+      # Assume a CGT discount is applicable 
+      
+      has_discount <-
+        if (min(Tot_CY_CG_amt.) == 0) {
+          Tot_CY_CG_amt. > Net_CG_amt.
+        } else {
+          and(Tot_CY_CG_amt. > Net_CG_amt.,
+              Tot_CY_CG_amt. > 0)
+        }
+      
+      out <- integer(max.length)
+      
+      out[has_discount] <-
+          as.integer(Net_CG_amt.[has_discount] * {{1 - new_rate} / {1 - old_rate} - 1})
+      
+      
+      
+      for (j in c("Net_CG_amt", "Tot_inc_amt", "Taxable_Income")) {
+        v <- .subset2(DT, j)
+        set(DT, 
+            j = j,
+            value = as.integer(v) + out)
+      }
+      
+      # Taxable Income cannot be negative
+      DT[, Taxable_Income := as.integer(pmax.int(Taxable_Income, 0L))]
+      
+      DT[]
+    }
+    extra_Net_CG_amt(sample_file)
+    # Need to update since the Taxable Income is now different.
+    income <- .subset2(sample_file, "Taxable_Income")
   }
   
   if (is.null(sapto_eligible)) {
@@ -201,6 +280,9 @@ model_income_tax <- function(sample_file,
     }
   }
   
+  
+
+  
   ordering <- NULL
   
   if (identical(key(sample_file), "Taxable_Income")) {
@@ -220,13 +302,20 @@ model_income_tax <- function(sample_file,
   
   # Check base tax
   if (!is.null(ordinary_tax_thresholds) || !is.null(ordinary_tax_rates)) {
-    tax_table2_fy <- tax_table2[fy_year == baseline_fy]
     
-    Thresholds <- ordinary_tax_thresholds %||% tax_table2_fy[["lower_bracket"]]
-    Rates <-  ordinary_tax_rates %||% tax_table2_fy[["marginal_rate"]]
+    # accessing the baseline rates is expensive if not necessary
+    if (is.null(ordinary_tax_thresholds) || is.null(ordinary_tax_rates)) {
+      tax_table2_fy <- tax_table2[fy_year == baseline_fy]
+      
+      Thresholds <- ordinary_tax_thresholds %||% tax_table2_fy[["lower_bracket"]]
+      Rates <-  ordinary_tax_rates %||% tax_table2_fy[["marginal_rate"]]
+    } else {
+      Thresholds <- ordinary_tax_thresholds
+      Rates <- ordinary_tax_rates
+    }
     
     if (length(Thresholds) != length(Rates)) {
-      stop("`ordinary_tax_thresholds` and `ordinary_tax_rates` have different lengths. ",
+      stop("`ordinary_tax_thresholds` and `ordinary_tax_rates` had different lengths. ",
            "Specify numeric vectors of equal length (or NULL).")
     }
     
@@ -234,6 +323,7 @@ model_income_tax <- function(sample_file,
       IncomeTax(income,
                 thresholds = Thresholds,
                 rates = Rates)
+    temp_budget_repair_levy. <- 0
   } else {
     tax_at <- lower_bracket <- marginal_rate <- NULL
     if ("ordering" %chin% names(input)) {
@@ -253,8 +343,6 @@ model_income_tax <- function(sample_file,
       and(input[["fy_year"]] %chin% c("2014-15", "2015-16", "2016-17"),
           income > 180e3) * 
       (0.02 * (income - 180e3))
-    
-    base_tax. <- base_tax. + temp_budget_repair_levy. 
   }
   
   WEIGHTj <- which(names(.dots.ATO) == "WEIGHT")
@@ -263,6 +351,7 @@ model_income_tax <- function(sample_file,
   }
   
   # If .dots.ATO  is NULL, for loops over zero-length vector
+  # Use integers since that's what tax forms use.
   for (j in which(vapply(.dots.ATO, FUN = is.double, logical(1)))) {
     if (j != WEIGHTj) {
       set(.dots.ATO, j = j, value = as.integer(.dots.ATO[[j]]))
@@ -430,9 +519,11 @@ model_income_tax <- function(sample_file,
                 if (is.null(medicare_levy_rate)) {
                   mr <- mt * (mb - ma) / mb
                   warning_if_misspecified("medicare_levy_rate")
+                  # nocov start
                   # alternative not reachable
                 } else stop("ERR # e59ed9845068f337d6653a7cc00401e1dbeeda7d. ",
                             "Please contact `grattan` package maintainer.") 
+                # nocov end
               }
             }
           }
@@ -591,13 +682,19 @@ model_income_tax <- function(sample_file,
     sapto_args <- mget(grep("^sapto_(?!eligible)", arguments, perl = TRUE, value = TRUE))
     
     if (all(vapply(sapto_args, is.null, FALSE))) {
-      
+      # 
+      .dASE <- .dots.ATO[(sapto_eligible),
+                         .SD, 
+                         .SDcols = c("Rptbl_Empr_spr_cont_amt", 
+                                     "Net_fincl_invstmt_lss_amt",
+                                     "Net_rent_amt",
+                                     "Rep_frng_ben_amt")]
       rebate_income_over_eligible <-
         rebate_income(Taxable_Income = income[sapto_eligible],
-                      Rptbl_Empr_spr_cont_amt = .dots.ATO[sapto_eligible][["Rptbl_Empr_spr_cont_amt"]],
-                      Net_fincl_invstmt_lss_amt = .dots.ATO[sapto_eligible][["Net_fincl_invstmt_lss_amt"]],
-                      Net_rent_amt = .dots.ATO[sapto_eligible][["Net_rent_amt"]],
-                      Rep_frng_ben_amt = .dots.ATO[sapto_eligible][["Rep_frng_ben_amt"]])
+                      Rptbl_Empr_spr_cont_amt = .dASE[["Rptbl_Empr_spr_cont_amt"]],
+                      Net_fincl_invstmt_lss_amt = .dASE[["Net_fincl_invstmt_lss_amt"]],
+                      Net_rent_amt = .dASE[["Net_rent_amt"]],
+                      Rep_frng_ben_amt = .dASE[["Rep_frng_ben_amt"]])
       
       sapto.[sapto_eligible] <-
         sapto(rebate_income = rebate_income_over_eligible, 
@@ -729,13 +826,17 @@ model_income_tax <- function(sample_file,
                                 fy_year = if (is.null(sbto_discount)) baseline_fy,
                                 tax_discount = sbto_discount)
     
-    if (.debug) {
-      return(data.table(income, base_tax., lito., lamington_offset., sapto., sbto., medicare_levy.))
-    }
-    
-    pmaxC(S4.10_basic_income_tax_liability - sbto., 0) +
+    new_tax <- 
+      pmaxC(S4.10_basic_income_tax_liability - sbto., 0) +
+      temp_budget_repair_levy. + 
       medicare_levy. +
       flood_levy.
+    
+    if (.debug) {
+      return(data.table(income, old_tax, base_tax., lito., lamington_offset., sapto., sbto., medicare_levy.))
+    }
+    
+    new_tax  
   }
   # new_tax2 <<- new_tax
   
@@ -750,7 +851,9 @@ model_income_tax <- function(sample_file,
     hutils::drop_col(sample_file, "new_taxable_income")
     sample_file[, "new_taxable_income" := new_taxable_income]
     
+    # nocov start
     if (anyNA(new_taxable_income) || identical(as.double(new_taxable_income), as.double(income))) stop("NAs: ", sum(is.na(new_taxable_income)), call. = FALSE)
+    # nocov end
     
     new_argument_vals <-
       argument_vals %>%
@@ -778,6 +881,7 @@ model_income_tax <- function(sample_file,
          "tax" = new_tax,
          "sample_file" = set(sample_file, j = "new_tax", value = new_tax),
          "sample_file.int" = set(sample_file, j = "new_tax", value = as.integer(new_tax)))
+  
 }
 
 
