@@ -4,8 +4,11 @@
 //' @name pmaxC
 //' @param x A numeric vector.
 //' @param a A single numeric value.
-//' @return The parallel maximum of the input values. \code{pmax0(x)} is shorthand for \code{pmaxC(x, 0)}, i.e. convert negative values in \code{x} to 0.
-//' @note This function will always be faster than \code{pmax(x, a)} when \code{a} is a single value, but can be slower than \code{pmax.int(x, a)} when \code{x} is short. Use this function when comparing a numeric vector with a single value.
+//' @return The parallel maximum of the input values. \code{pmax0(x)} is
+//'  shorthand for \code{pmaxC(x, 0)}, i.e. convert negative values in \code{x} to 0.
+//' @note This function will always be faster than \code{pmax(x, a)} when \code{a} is
+//'  a single value, but can be slower than \code{pmax.int(x, a)} when \code{x} is short.
+//'  Use this function when comparing a numeric vector with a single value.
 //' @export pmaxC
 
 #include <Rcpp.h>
@@ -79,18 +82,38 @@ NumericVector pmaxC(NumericVector x, double a) {
 NumericVector pmax0(NumericVector x) {
   int n = x.length();
   NumericVector out(n);
-  
+ 
   for (int i = 0; i < n; ++i) {
     double xi = x[i];
     if (xi > 0) {
       out[i] = xi;
     }
   }
-  
   return out;
 }
 
+// [[Rcpp::export]]
+NumericVector pmaxIPnum0(NumericVector x) {
+  int n = x.length();
+  
+  for (int i = 0; i < n; ++i) {
+    if (x[i] < 0) {
+      x[i] = 0;
+    }
+  }
+  return x;
+}
 
-
+// [[Rcpp::export]]
+IntegerVector pmaxIPint0(IntegerVector x) {
+  int n = x.length();
+  
+  for (int i = 0; i < n; ++i) {
+    if (x[i] < 0) {
+      x[i] = 0;
+    }
+  }
+  return x;
+}
 
 
