@@ -15,10 +15,13 @@ test_that("Wage growth is higher for extreme salaries", {
   skip_on_cran()
   skip_if_not_installed("taxstats")
   skip_if_not_installed("dplyr")
+  skip_on_travis()
   library(taxstats)
   library(dplyr)
   skip_if_not(exists("get_sample_files_all"))
-  try(sample_files_all <- get_sample_files_all())
+  if (!exists("sample_files_all")) {
+    try(sample_files_all <- get_sample_files_all())
+  }
   skip_if_not(exists("sample_files_all"))
   
   extreme_tile <- sample(c(1:20, 80:100), size = 1)
@@ -87,7 +90,10 @@ test_that("Differentially uprated wage growth is *up*", {
 })
 
 test_that("Less than 0.1% of individuals move more than one percentile over 10 years", {
-  skip_if_not_installed("taxstats") 
+  skip_if_not_installed("taxstats")
+  skip_if_not_installed("dplyr")
+  library(dplyr)
+  library(taxstats)
   skip_on_cran()
   prop_move <- 
     sample_file_1314 %>%
@@ -104,7 +110,8 @@ test_that("differential wage inflator is mean-preserving", {
   skip_if_not_installed("taxstats") 
   skip_on_cran()
   library(magrittr)
-  salaries_1314 <- sample_file_1314$Sw_amt
+  library(taxstats)
+  salaries_1314 <- sample_file_1314[["Sw_amt"]]
 
   salaries_1314_vanilla <- wage_inflator(salaries_1314, from_fy = "2013-14", to_fy = "2015-16")
   salaries_1314_differe <- differentially_uprate_wage(salaries_1314, from_fy = "2013-14", to_fy = "2015-16")
