@@ -4,6 +4,8 @@ test_that("medicare_levy does not respond to invalid", {
   expect_error(medicare_levy(income = 123, fy.year = c("2015-16", "2015-17")))
   expect_error(medicare_levy(income = 123, fy.year = "2014-15", family_status = "couple"))
   expect_error(medicare_levy(income = 123, fy.year = "2015-16", sapto.eligible = FALSE, Spouse_income = 1e3, n_dependants = 0, family_status = "individual"))
+  
+  
 })
 
 test_that("medicare_levy monotonic", {
@@ -54,6 +56,8 @@ test_that("medicare_levy returns known values", {
   expect_equal(medicare_levy(income = 52e3, fy.year = "2015-16", sapto.eligible = TRUE , Spouse_income = 1e3, n_dependants = 2, family_status = "family"), 0)
   
   expect_equal(medicare_levy(20e3, "2004-05", sato = TRUE, pto = FALSE), 0)
+  expect_equal(medicare_levy(20e3, "2004-05", sato = NULL, pto = FALSE), 0)
+  expect_equal(medicare_levy(20e3, "2004-05", sato = TRUE), 0)
   expect_equal(medicare_levy(16e3, "2001-02"), 240)
   expect_equal(medicare_levy(16e3, "2001-02", sapto.eligible = TRUE), 0)
 })
@@ -106,10 +110,51 @@ test_that("new_medicare_levy matches", {
     .[fy_year == "2013-14"] %>%
     setnames(old = "sapto", new = "switches")
   
+  expect_equal(new_medicare_levy(par_tbl)(income = 23e3,
+                                          switch = FALSE, 
+                                          Spouse_income = 750,
+                                          n_dependants = 0,
+                                          family_status = "family"), 
+               medicare_levy(income = 23e3,
+                             fy.year = "2015-16",
+                             sapto.eligible = TRUE,
+                             Spouse_income = 750,
+                             n_dependants = 0,
+                             family_status = "family"))
+  expect_error(new_medicare_levy(par_tbl)(25e3,  
+                                          family_status = "individual",
+                                          switch = FALSE,
+                                          Spouse_income = 100),
+               regexp = "Whenever `Spouse_income` is positive,")
+  expect_error(new_medicare_levy(par_tbl)(rep(25e3, 2),
+                                          family_status = rep("individual", 2),
+                                          switch = TRUE,
+                                          Spouse_income = 100),
+               regexp = "Whenever `Spouse_income` is positive,")
+  
   expect_error(new_medicare_levy(parameter_table = as.data.frame(par_tbl)))
-  expect_error(new_medicare_levy(parameter_table = as.data.frame(par_tbl) %>% select(-taper)))
+  expect_error(new_medicare_levy(parameter_table = hutils::drop_cols(par_tbl, "taper")),
+               regexp = "parameter_table must contain certain columns")
+  
   
-  expect_equal(new_medicare_levy(par_tbl)(income = 23e3, switch = FALSE, Spouse_income = 750, n_dependants = 0, family_status = "family"), 
-               medicare_levy(income = 23e3, fy.year = "2015-16", sapto.eligible = TRUE , Spouse_income = 750, n_dependants = 0, family_status = "family"))
+})
+
+
+test_that("Medicare levy C++ constant", {
+  expect_equal(MedicareLevySaptoYear(47900, 15e3, 1L, TRUE, 2017), 
+               medicare_levy(47900,
+                             fy.year = "2016-17",
+                             sapto.eligible = TRUE,
+                             Spouse_income = 15e3,
+                             n_dependants = 1L,
+                             family_status = "family"))
+  
+  expect_equal(MedicareLevySaptoYear(47900, 15e3, 1L, TRUE, 2019), 
+               medicare_levy(47900,
+                             fy.year = "2018-19",
+                             sapto.eligible = TRUE,
+                             Spouse_income = 15e3,
+                             n_dependants = 1L,
+                             family_status = "family"))
 })
 
