@@ -11,7 +11,18 @@ test_that("grattan functions and pmax pmin give identical results", {
   expect_equal(pminV(x, y), pmin(x, y))
   expect_equal(pmax3(x, y, z), pmax(x, pmax(y, z)))
   
+  expect_equal(pmin0(c(-1, 0, 1)), 
+               pmin(c(-1, 0, 1), 0))
+  
   expect_error(pmax3(1, 2, 3:4))
   expect_error(pmaxV(1, 1:2))
+  expect_error(pminV(1, 1:2))
+})
+
+test_that("pmaxIPint0", {
+  expect_equal(pmaxIPint0(-2:2),
+               hutils::if_else(-2:2 > 0, -2:2, 0L))
+  expect_equal(pmaxIPint0(1:5),
+               1:5)
 })
 
